import os
import pytest
import responses
from spree import Spree


@pytest.fixture
def fp():
    def wrapper(rel_path):
        "return the full path of given rel_path"
        return os.path.abspath(
            os.path.join(
                os.path.dirname(__file__),
                rel_path
            )
        )
    return wrapper


@pytest.fixture
def spree():
    return Spree('http://mystore/api', 'some-key')


@pytest.yield_fixture
def resp(fp):
    rsps = responses.RequestsMock(False)
    rsps.start()

    "Products"
    rsps.add(
        responses.GET,
        'http://mystore/api/products',
        body=open(fp('responses/products.json'), 'r').read()
    )
    rsps.add(
        responses.GET,
        'http://mystore/api/products/2',
        body=open(fp('responses/product-2.json'), 'r').read()
    )

    "Orders"
    rsps.add(
        responses.GET,
        'http://mystore/api/orders',
        body=open(fp('responses/orders.json'), 'r').read()
    )
    rsps.add(
        responses.GET,
        'http://mystore/api/orders/R123456789',
        body=open(fp('responses/order-R123456789.json'), 'r').read()
    )

    "StockItems"
    rsps.add(
        responses.GET,
        'http://mystore/api/stock_locations/1/stock_items',
        body=open(fp('responses/stock_items.json'), 'r').read()
    )
    rsps.add(
        responses.GET,
        'http://mystore/api/stock_locations/1/stock_items/4',
        body=open(fp('responses/stock_item-4.json'), 'r').read()
    )

    "Variants"
    rsps.add(
        responses.GET,
        'http://mystore/api/products/ruby-on-rails-jr-spaghetti/variants',
        body=open(fp('responses/variant-ROR-00013.json'), 'r').read()
    )

    "Stock Locations"
    rsps.add(
        responses.GET,
        'http://mystore/api/stock_locations',
        body=open(fp('responses/stock_locations.json'), 'r').read()
    )
    rsps.add(
        responses.GET,
        'http://mystore/api/stock_locations/1',
        body=open(fp('responses/stock_location-1.json'), 'r').read()
    )

    yield rsps

    rsps.stop()
    rsps.reset()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_spree
----------------------------------

Tests for `spree` module.
"""

import pytest 	# noqa


def test_products_all(resp, spree):
    "test fetching all products"
    products = spree.product.find()
    assert products.count == 16
    test_product = products[1]
    assert test_product['name'] == 'Ruby on Rails Bag'
    assert test_product['master']['sku'] == 'ROR-00012'
    assert test_product['description'] == 'Et illo et' \
        ' voluptatum corporis. Fugiat atque optio voluptates' \
        ' placeat ut odio. Iusto totam molestiae tenetur eaque' \
        ' molestias. Occaecati sunt sit magni voluptatibus qui ipsum.'
    assert test_product['price'] == '22.99'
    assert test_product['display_price'] == '$22.99'
    assert test_product['available_on'] == '2016-06-27T11:10:17.000Z'
    assert test_product['slug'] == 'ruby-on-rails-bag'
    assert test_product['shipping_category_id'] == 1


def test_finding_one_product(resp, spree):
    "test fetching product with respect to id"
    test_product = spree.product.get(2)
    assert test_product['name'] == 'Ruby on Rails Bag'
    assert test_product['master']['sku'] == 'ROR-00012'
    assert test_product['description'] == 'Et illo et' \
        ' voluptatum corporis. Fugiat atque optio voluptates' \
        ' placeat ut odio. Iusto totam molestiae tenetur eaque' \
        ' molestias. Occaecati sunt sit magni voluptatibus qui ipsum.'
    assert test_product['price'] == '22.99'
    assert test_product['display_price'] == '$22.99'
    assert test_product['available_on'] == '2016-06-27T11:10:17.000Z'
    assert test_product['slug'] == 'ruby-on-rails-bag'
    assert test_product['shipping_category_id'] == 1


def test_orders_all(resp, spree):
    "test fetching all orders"
    test_orders = spree.order.find()
    test_order = test_orders[0]
    assert test_order['number'] == 'R123456789'
    assert test_order['item_total'] == '15.99'
    assert test_order['state'] == 'complete'
    assert test_order['shipment_state'] == 'pending'
    assert test_order['payment_total'] == '0.0'
    assert test_order['token'] == 'aAoojFjB1_PCBeLR1VPnIg'


def test_finding_one_order(resp, spree):
    "test fetching order with respect to order_id"
    test_order = spree.order.get('R123456789')
    assert test_order['number'] == 'R123456789'
    assert test_order['item_total'] == '15.99'
    assert test_order['state'] == 'complete'
    assert test_order['shipment_state'] == 'pending'
    assert test_order['payment_total'] == '0.0'
    assert test_order['token'] == 'aAoojFjB1_PCBeLR1VPnIg'


def test_finding_stock_locations(resp, spree):
    "test finding stock locations"
    stock_locations = spree.stock_locations.find()

    # get a specific stock location with id
    stock_location = spree.stock_locations.get(1)
    assert stock_location == stock_locations[0]


def test_stock_items_all(resp, spree):
    "test fetching all stock items"
    stock_item = spree.get_stock_item(1)
    test_stock_items = stock_item.find()
    test_stock_item = test_stock_items[0]
    assert test_stock_item['count_on_hand'] == 10
    assert test_stock_item['stock_location_id'] == 1
    assert test_stock_item['variant_id'] == 4


def test_finding_one_stock_item(resp, spree):
    "test fetching all stock items"
    stock_item = spree.get_stock_item(1)
    test_stock_item = stock_item.get(4)
    assert test_stock_item['count_on_hand'] == 10
    assert test_stock_item['stock_location_id'] == 1
    assert test_stock_item['variant_id'] == 4


def test_variant_via_permalink(resp, spree):
    "test fetching variant by product permalink"
    variant = spree.get_variant('ruby-on-rails-jr-spaghetti')
    test_variants = variant.find()
    test_variant = test_variants[0]
    assert test_variant['sku'] == 'ROR-00013'
    assert test_variant['price'] == '19.99'
    assert test_variant['cost_price'] == '17.0'



# -*- coding: utf-8 -*-
import requests
from .exceptions import ResourceNotFound


class Spree(object):

    def __init__(self, url, api_key):
        self.url = url
        self.api_key = api_key
        self.session = requests.Session()
        self.session.headers['X-Spree-Token'] = api_key

    @property
    def product(self):
        return Product(connection=self)

    @property
    def order(self):
        return Order(connection=self)

    def get_stock_item(self, location_id):
        return StockItem(location_id, connection=self)

    def get_variant(self, product_id):
        return Variant(product_id, connection=self)

    @property
    def stock_locations(self):
        return StockLocation(connection=self)


class Pagination(object):
    def __init__(self, data, items_attribute, resource, filters=None):
        self.data = data
        self.items = data[items_attribute]
        self.current_index = -1
        self.resource = resource
        self.filters = filters

    @property
    def count(self):
        return int(self.data['count'])

    @property
    def page(self):
        return int(self.data['current_page'])

    @property
    def pages(self):
        return int(self.data['pages'])

    @property
    def has_next(self):
        return self.pages > self.page

    def next_page(self):
        if self.has_next:
            return self.resource.find(
                page=self.page + 1, filters=self.filters)

    def __iter__(self):
        return self

    def __getitem__(self, index):
        return self.items[index]

    def __setitem__(self, index, value):
        self.items[index] = value

    def next(self):
        if self.current_index < self.count-1:
            self.current_index += 1
            return self.items[self.current_index]
        else:
            raise StopIteration()


class Resource(object):
    """
    A base class for all Resources to extend
    """

    def __init__(self, connection, per_page=25):
        self.connection = connection
        self.per_page = per_page

    @property
    def url(self):
        return self.connection.url + self.path

    @classmethod
    def validate_response(cls, response):
        if response.status_code == 404:
            raise ResourceNotFound('Resource not found.')
        response.raise_for_status()

    def load_payload(self, data):
        return data

    def find(self, page=1, filters=None):
        """
        Find all records that respect given filters

        :param filters: Dictionary whose key-value pairs are in
        for of: {
            'q[name_of_parameter_identifier]': 'some-value',
        }
        Reference: https://github.com/ernie/ransack/wiki/Basic-Searching
        :return: Paginated list of response
        """
        params = filters or {}
        params.update({
            'page': page,
            'per_page': self.per_page,
        })
        response = self.connection.session.get(self.url, params=params)
        self.validate_response(response)
        return Pagination(
            response.json(),
            self.item_attribute,
            resource=self,
            filters=filters,
        )

    def get(self, id):
        "Fetch a record with given id"
        path = self.url + '/%s' % id
        response = self.connection.session.get(path)
        self.validate_response(response)
        return response.json()

    def create(self, data):
        "create a record with the given data"
        payload = self.load_payload(data)
        response = self.connection.session.post(self.url, data=payload)
        self.validate_response(response)
        return response.json()

    def update(self, id, data):
        "update the record with given data"
        path = self.url + '/%d' % id
        payload = self.load_payload(data)
        response = self.connection.session.put(path, data=payload)
        self.validate_response(response)
        return response.json()

    def delete(self, id):
        "delete a given record"
        path = self.url + '/%d' % id
        response = self.connection.session.delete(path)
        self.validate_response(response)
        return response.json()


class Product(Resource):
    """
    A product Resource class
    """

    path = '/products'
    item_attribute = 'products'

    def load_payload(self, data):
        payload = {
                'product[name]': data['name']
            }
        if 'price' in data:
            payload['product[price]'] = data['price']
        if 'shipping_category_id' in data:
            payload['product[shipping_category_id]'] = \
                data['shipping_category_id']
        if 'sku' in data:
            payload['product[sku]'] = data['sku']
        if 'description' in data:
            payload['product[description]'] = data['description']
        if 'display_price' in data:
            payload['product[display_price]'] = data['display_price']
        if 'available_on' in data:
            payload['product[available_on]'] = data['available_on']
        if 'meta_description' in data:
            payload['product[meta_description]'] = data['meta_description']
        if 'meta_keywords' in data:
            payload['product[meta_keywords]'] = data['meta_keywords']
        if 'weight' in data:
            payload['product[weight]'] = data['weight']
        if 'height' in data:
            payload['product[height]'] = data['height']
        if 'width' in data:
            payload['product[width]'] = data['width']
        if 'depth' in data:
            payload['product[depth]'] = data['depth']
        if 'cost_price' in data:
            payload['product[cost_price]'] = data['cost_price']

        return super(Product, self).load_payload(payload)


class Order(Resource):
    """
    An order Resource class
    """

    path = '/orders'
    item_attribute = 'orders'


class StockLocation(Resource):
    """
    A stock location Resource class
    """

    path = '/stock_locations'
    item_attribute = 'stock_locations'


class StockItem(Resource):
    """
    A stock item Resource class
    """

    item_attribute = 'stock_items'

    def __init__(self, location_id, *args, **kwargs):
        super(StockItem, self).__init__(*args, **kwargs)
        self.location_id = location_id

    @property
    def path(self):
        return '/stock_locations/%d/stock_items' % self.location_id

    def load_payload(self, data):
        payload = {}
        if 'count_on_hand' in data:
            payload['stock_item[count_on_hand]'] = \
                data['count_on_hand']
        if 'force' in data:
            payload['stock_item[force]'] = data['force']
        return super(StockItem, self).load_payload(payload)


class Variant(Resource):
    """
    A variant item Resource class
    """

    item_attribute = 'variants'

    def __init__(self, product_id, *args, **kwargs):
        super(Variant, self).__init__(*args, **kwargs)
        self.product_id = product_id

    @property
    def path(self):
        return '/products/%s/variants' % self.product_id
